function [bb,cc,F,z0]=acor2tf(ro,n,tol)
%ACOR2TF Transfer functions computed out of autocorreletion function
%
%   Syntax
%   [bb,cc,F,z0]=acor2tf(ro,n)
%   [bb,cc,F,z0]=acor2tf(ro,n,tol)
%
%   Description
% [bb,cc,F,z0]=acor2tf(ro,n) calculates all possible transfer functions (bb,cc)
%   for the autocorrelation function ro, taking m >= n+1 values of ro. n is the
%   estimated filter degree. The solutions are stored by rows in the
%   matrices (bb,cc). The first rows define the minimal phase solution. In
%   addition the transfer function F, containing all possible zeros, and
%   the roots of this function are returned.
%
% [bb,cc,F,z0]=acor2tf(ro,n,tol) allows to define a tolerance value for 
%   multiple root extraction;  by default: tol = sqrt(eps); 
%   eps = minimal value of the computer system.
%
% Remerk: Problem with Calculation of multiple roots may occur;
%        The function 'multroots' published by  Z. Zeng, Chicago, may be
%        used for the calculation of multiple roots.

%   References: [DSV1, chap. 5.6.2, p. 410]
%
%   See also DSV-Lib: imp2tf, extphase, circlemean
%            Matlab:  conv, roots

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Hans W. Schuessler and Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,3, nargin, 'struct'))
if nargin==3, reps=tol; else reps=sqrt(eps); end
% if length(ro)<2*n, error('acor2tf: length of ro < 2*n'); end
s = ro; s(1)=s(1)/2;
[bs,cs] = imp2tf(s);
if length(cs)<n+1, warning('DSVlib:acor2tf',['approx. filter order <n; '...
        'use longer autocor. fun.']); end 
bs_=fliplr(bs); cs_=fliplr(cs);
F=conv(bs,cs_) + conv(bs_,cs);
 z0=roots(F);
%[z0,result]=multroots(F); % Procedure according to Z. Zeng, Chicago 
%nz=length(z0);
% Build minimum-phase system
absz=abs(z0);
int1=find(absz<1-reps);
int3=find(abs(absz-1)<reps);
z1=z0(int3);
nz1=length(z1);
rep=z1(:,ones(1,nz1));
gap=2*eye(nz1) + abs(rep - rep.');
z=[]; i=0;
while ~isempty(gap)
    i=i+1;
    [gapm,int]=min(gap(:,1));
    z=[z;circlemean(z1(gap(:,1)==2),z1(int))]; %#ok<AGROW>
    gap(:,1)=[]; gap(:,int-2*i+1)=[];
end
z=[z;z0(int1)];
b=real(poly(z)); 
% Get additional transfer functions
[bb,cc] = extphase(b,cs);
%% [EOF] - acor2tf

