function [bb,cc] = extphase(b,c,tol)
%EXTPHASE:  Transfer function  with non-minimal-phase 
%   In addition to the given minimal phase solution, possible
%   additional non-minimal-phase transfer function of the same
%   degree are calculated by mirroring of the zeros within the unit
%   circle.
%
%   Syntax
%   [bb,cc] = extphase(b,c)
%   [bb,cc] = extphase(b,c,tol)
%
%   Description
% [bb,cc] = extphase(b,c)calculates all possible transfer function (bb,cc)
%   for the given minimal phase transfer function. The solutions are stored 
%   by rows in the matrices (bb,cc). The first rows define the minimal 
%   phase solution, again. 
% [bb,cc] = extphase(b,c,tol) allows  to define a tolerance value for 
%   multiple root extraction;  by default: tol = sqrt(eps); with
%   eps = minimal value of the computer system.
%
%   Remerk: Problem with Calculation of multiple roots may occur;
%        The function 'multroots' published by  Z. Zeng, Chicago, may be
%        used for the calculation of multiple roots.
%
%   References: [DSV1, chap. 5.6.2, p. 409]
%
%   See also DSV-Lib: acor2tf, acorimpresp, unitmirror
%            Matlab:  poly, polyval

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors:  Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,3, nargin, 'struct'))
if nargin==3, reps=tol; else   reps=sqrt(eps); end
bb=b;cc=c;
bb=bb*polyval(cc,1)/polyval(bb,1);
z0=roots(b);  absz=abs(z0); zz=z0;
int1=find(absz<1-reps);
int2=find(absz>1+reps);            % zeros outside of unit cycle
int3=find(abs(absz-1)<reps);
%
if ~isempty(int2), 
  warning('DSVlib:extphase',' Basis system is not minimal-phase'); end
%
if isempty(int1),
  warning('DSVlib:extphase',' No zeros to mirror available');
  return
end
z20=z0([int2,int3]);
z1=z0(int1);
z1=z1(imag(z1)>=-reps);
n1=length(z1);
m3=2^n1-1;
mat=double(dec2bin(1:m3))-48;
for i=1:m3
  z2=z20;
  for n=1:n1
    z=z1(n);
    if mat(i,n), z=unitmirror(z); end
    z2=[z2;z];
    if ~isreal(z), z2=[z2;conj(z)]; end
  end
  zz=[zz,z2];
  b =real(poly(z2));
  b =b*polyval(cc,1)/polyval(b,1);
  bb=[bb;b];
end
%% [EOF] - extphase
