function [b,c] = imp2tf(hl,tol)
%IMP2TF Rational transfer function defined by impulse response
%   The measured impulse response is transformed back to an equivalent
%   rational transfer function
%
%   Syntax
%   [b,c] = imp2tf(hl)
%   [b,c] = imp2tf(hl,tol)
%
%   Description
% [b,c] = imp2tf(hl)calculates the transfer function (b,c) in dependence of
%   the given impulse response of the system.
% [b,c] = imp2tf(hl,tol) allows  to define a tolerance value for 
%   the determination of the rank of the "Hankel"-matrix ;  
%   by default: tol = 10^3*eps; with
%   eps = minimal value of the computer system.
%

%   References: [DSV1, chap. 5.3.2]
%
%   See also DSV-Lib: acor2tf
%            Matlab:  hanel, toeplitz, rank

% 
% Literatur: Schuessler, H.W.; DSV1, Kap. 5.3.2

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Hans W. Schuessler 
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1, 2, nargin, 'struct'))
if nargin<2, tol=1e3*eps; end
hl = hl(:); l =length(hl);
% Generate Hankelsche Matrix ; n = System order:
l1 = floor (l/2); S = hankel(hl(2:l1),hl(l1:2*l1));
if nargin == 1,  n = rank(S);
else             n = rank(S,tol);
end
% Denominator coefficients:
Hc= S(1:n,1:n); h = hl(n+2:2*n+1);
c= -Hc\h; c= [1 fliplr(c')];
% Numerator coefficients:
Hb = toeplitz(hl(1:n+1),[hl(1) zeros(1,n)]);
b = (Hb * c')' ;
%% [EOF] - imp2tf
