function [y,zss,nov] = fixdf1sos(soc,s,v,zs,w,rmode,over)
%FIXDF1SOS Quantized digital SOS-filter in df1-form
%   Realization
%   - one internal quantizer after addition defined by rmode,
%   - no quantization after final multiplication s(nb+1)
%   - overflow check after addition and at output
%   - quantization in front of scaling multiplication for full
%     multipliers only
%   - state variables realized sparately for numerators and denominators
%   To be able to calculate (only) the internal noise, there is 
%   no extra quantization of the input signal implemented. 
%   If necessary, the input signal has to quantized before calling 
%   the function.
%
%   Syntax
%   y  = fixdf1sos(soc,s,v)
%   y  = fixdf1sos(soc,s,v,zs)
%   y  = fixdf1sos(soc,s,v,zs,w)
%   y  = fixdf1sos(soc,s,v,zs,w,rmode)
%   y  = fixdf1sos(soc,s,v,zs,w,rmode,over)
%   [y,zss,nov]=fixdf1sos(soc,s,v,...)
%
%   Description
% y = fixdf1sos(soc,s,v) The input signal v is filtered with
%   start state values z(:) = 0,  w=16 bit, rmode='round' and
%   overflow characterisic over='sat'. The output sequence y is 
%   returned.
%   soc MATLAB SOS filter coefficient array for nb sections
%   s   vector with scaling factors (length: 1, 2 or nb+1)
%       s(1)    input scaling factor
%       s(i+1)  block scaling factor between sections
%       s(nb+1) output scaling factor
%   v   input signal
%   zs  state variables (nb x 2); default: zs=[]
%   w   internal wordlength; default: w = 16
%   rmode control quantization; default: rmode='round' (TR)
%   over control overflow mode; default: over = 'sat'
%   For a further description the parameters w, rmode and over 
%   see function fixquant(.).
%
% y = fixdf1sos(soc,s,v,[],12,'convergent','wrap') In addition
%   the internal wordlength is set to 12 bit, quantization is done by
%   the convergent mode and overflows are handled by the wrap
%   characteristic. The state variables are set by default.
%   Remark: All intermediate parameters have to be set!
%
% [y,zss,nov] = fixdf1sos(soc,s,v,...) computes optional output values
%   zss  return of state variables
%   nov  overflow counter array (nb+1 x 2) 
%
%   References: [DSV1, chap. 5.1 and 5.2]
%               [Deh03]
%
%   See also DSV-Lib: df2sos, df2tsos, fixquant
%            Matlab: sosfilt 
% 

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,7, nargin, 'struct'))
if (nargin<7), over ='sat';    end
if (nargin<6), rmode='nearest';end
if (nargin<5), w  = 16; end
if (nargin<4), zs = []; end
nsec = size(soc,1);  ns   = length(s);
if ns < 2, s=[s,ones(1,1)]; ns=2; end
if ns < nsec+1,s = [s(1),ones(1,nsec+1-ns),s(2)]; end 
% reset state variables
if(isempty(zs)), zs = zeros(nsec,4); end
noc = zeros(nsec,2);
xx = v;
for isec = 1:nsec
    if s(isec)~=1.,
      [xx,noc(isec,2)] = fixquant( xx*s(isec),w,'nearest', over);
    end
    co  = [soc(isec,1:3),-soc(isec,5:6)];
    if(soc(isec,4)~=1), 
        error('SOSFXDF1: denominator coefficients?');
    end
    [xx,zs(isec,:),noc(isec,1)] = fx1fi(co,xx,zs(isec,:),w,rmode,over);
end
xx = xx * s(nsec+1);
% set output parameter
y  = xx;
if (nargout>1), zss = zs; end
if (nargout>2), 
nov = noc;
else
    if( max(noc(:))>0),
        warning('DSVLib:fixdf1sos',' Overflow detected');
    end
end

% ----------------------------------------------------------
function [y,zi,noc] = fx1fi(co,v,zi,w,rmode,over)
%   zi = state variable x11 x21 || x21 x22
x = zi.';
noc = 0.;
LEN = length(v);
n = 0;  y=zeros(size(v));
while n < LEN,
     n = n + 1;
     x=[v(n);x(1:4)];
     [u,inoc] = fixquant((co * x),w,rmode,over);
     y(n) = u;
     x(3) = u;
     noc = noc + inoc;
end
zi = (x(1:4)).';
%% [EOF] - fixdf1sos
