function [y,zss,nov] = fixdf2sos(soc,s,v,zs,w,rmode,over)
%FIXDF2SOS Quantized digital SOS-filter in df2-form
%   Realization
%   - one internal quantizer after addition defined by 'rmode',
%   - quantization of the output signal after summation 
%     of the last numerator multiplications, 
%   - no quantization after final multiplication s(nb+1),
%   - overflow check after addition, scaling multiplication 
%     and at output,
%   - quantization in front of  scaling multiplication for full
%     multipliers only.
%   For quantization of the input signal, see fixdf1sos(.).
%
%
%   Syntax
%   y  = fixdf2sos(soc,s,v)
%   y  = fixdf2sos(soc,s,v,zs)
%   y  = fixdf2sos(soc,s,v,zs,w)
%   y  = fixdf2sos(soc,s,v,zs,w,rmode)
%   y  = fixdf2sos(soc,s,v,zs,w,rmode,over)
%   [y,zss,nov]=fixdf2sos(soc,s,v,...)
%
%   Description
% y = sosfxdf2(soc,s,v) The input signal v is filtered with
%   start state values z(:) = 0,  w=16 bit, rmode='round' and
%   overflow characterisic over='sat'. The output sequence y is 
%   returned.
%   soc MATLAB SOS filter coefficient array for nb sections
%   s   vector with scaling factors
%       s(1)    input scaling factor
%       s(i+1)  block scaling factor between sections
%       s(nb+1) output scaling factor
%   v   input signal
%   zs  state variables (nb x 2); default: zs=[]
%   w   internal wordlength; default: w = 16
%   rmode control quantization; default: rmode='nearest' (TR)
%   over control overflow mode; default: over = 'sat'
%   For a further description the parameters w, rmode and over 
%   see fixquant(.).
%
% y = sosfxdf2(soc,s,v,[],12,'convergent','wrap') In addition
%   internal wordlength is set to 12 bit, quantization is done by
%   the convergent mode and overflows are handled by the wrap
%   characteristic. The state variables are set by default.
%   Remark: All intermediate parameters have to be set!
%
%
% [y,zss,nov] = fixdf2sos(soc,s,v,...) computes optional output values
%   zss  return of state variables
%   nov overflow counter array (nb+1 x 2) 
% 
%   Examples: see DSV1 Section 7.1.7.5
%
%   References: [DSV1, chap. 5.1 and 5.2]
%               [Deh03]
%
%   See also DSV-Lib: df1sos, df2tsos, fixquant
%            Matlab: sosfilt 

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,7, nargin, 'struct'))
%
if (nargin<7), over ='sat';    end
if (nargin<6), rmode='nearest';end
if (nargin<5), w  = 16; end
if (nargin<4), zs = []; end
nsec = size(soc,1); ns=length(s);
if ns < 2, s=[s,ones(1,1)]; ns=2; end
if (ns < nsec+1), s = [s(1),ones(1,nsec+1-ns),s(2)]; end
% reset state variables
if(isempty(zs)), zs = zeros(nsec,2); end
noc = zeros(nsec,2);
%%
xx = v;
for isec = 1:nsec
    xx = xx * s(isec);   % no quantization after scaling mult.
    b  = soc(isec,1:3);
    if soc(isec,4)~=1, 
        error('FIXDF2SOS: denominator coefficients?')
    end
    a = soc(isec,4:6);
    [xx,zs(isec,:),noc(isec,1)] = fx2fi(b,a,xx,zs(isec,:),w,rmode,over);
    % check for one or shift multipliers
    if(isec ~=nsec && s(isec+1)~=1),
       if(ispower2(s(isec+1))),  % overflow check only
          [xx,noc(isec+1,2)] = fxquant( xx, w, 'none', over);
       else
          [xx,noc(isec+1,2)] = fxquant( xx, w, rmode, over);
       end
    end
    if isec==nsec,
       [xx,noc(nsec,2)] = fixquant( xx, w,rmode, over);
    end
end
xx = xx * s(nsec+1);
% set output parameter
y  = xx;
if (nargout>1), zss = zs; end
if (nargout>2), nov = noc;
else
    if( max(noc(:))>0),
        warning('DSVLib:Overflow',' detected in FIXDF2SOS');
    end
end

%% Helper Functions----------------------------------------
function [y,zi,noc] = fx2fi(b,a,v,zi,w,rmode,over)
%FX2FI: SOS - second order section in df2 form
a = a(2:3);
x = zi.';
noc = 0.;
LEN = length(v);
n = 0;
y=zeros(size(v));
while n < LEN,
     n = n + 1;
     [u,inoc] = fixquant((v(n) - a * x),w,rmode,over);
     y(n) = b*[u;x];
     x = [u;x(1)];
     noc = noc + inoc;
end
zi = x.';
%
% ----------------------------------------------------------------------
function lv = ispower2 (x)
% ISPOWER2:  test for power of 2
xx = log2(x);
if((floor(xx)-xx)== 0), 
    lv = 1; 
else
    lv = 0;
end
%% [EOF] - fixdf2sos
