function [y,zss,nov] = fixdf2tsos(soc,s,v,zs,w,rmode,over)
%FIXDF2TSOS Quantized digital SOS-filter in df2t-form
%   Realization
%   - three internal quantizers after addition, controlled by
%     parameter rmode
%   - no quantization after final multiplication s(nb+1)
%   - overflow check after addition, scaling multiplication
%     and at output
%   - quantization after scaling multipliers always by
%     'nearest' mode
%   For quantization of the input signal, see fixdf1sos(.).
%
%   Syntax
%   y  = fixdf2tsos(soc,s,v)
%   y  = fixdf2tsos(soc,s,v,zs)
%   y  = fixdf2tsos(soc,s,v,zs,w)
%   y  = fixdf2tsos(soc,s,v,zs,w,rmode)
%   y  = fixdf2tsos(soc,s,v,zs,w,rmode,over)
%   [y,zss,nov]=fixdf2tsos(soc,s,v,...)
%
%   Description
% y = fixdf2tsos(soc,s,v) The input signal v is filtered with
%   start state values z(:) = 0,  w=16 bit, rmode='round' and
%   overflow characterisic over='sat'. The output sequence y is 
%   returned.
%   soc MATLAB SOS filter coefficient array for nb sections
%   s   vector with scaling factors
%       s(1)    input scaling factor
%       s(i+1)  block scaling factor between sections
%       s(nb+1) output scaling factor
%   v   input signal
%   zs  state variables (nb x 2); default: zs=[]
%   w   internal wordlength; default: w = 16
%   rmode control quantization; default: rmode='round' (TR)
%   over control overflow mode; default: over = 'sat'
%   For a further description the parameters w, rmode and over 
%   see fixquant(.).
%
% y = fixdf2tsos(soc,s,v,[],12,'convergent','wrap') In addition
%   internal wordlength is set to 12 bit, quantization is done by
%   the convergent mode and overflows are handled by the wrap
%   characteristic. The state variables are set by default.
%   Remark: All intermediate parameters have to be set!
%
% [y,zss,nov] = fixdf1sos(soc,s,v,...) computes optional output values
%   zss  return of state variables
%   nov  overflow counter array (nb+1 x 2) 
%
%   References: [DSV1, chap. 5.1 and 5.2]
%               [Deh03]
%
%   See also DSV-Lib: df1sos, df2sos, fixquant
%            Matlab: sosfilt 

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,7, nargin, 'struct'))
if (nargin<7), over = 'sat';  end
if (nargin<6), rmode='nearest';end
if (nargin<5), w    = 16; end
if (nargin<4), zs = [];   end
nsec = size(soc,1); ns=length(s);
if ns < 2, s=[s,ones(1,1)]; ns=2; end
if (ns < nsec+1), s = [s(1),ones(1,nsec+1-ns),s(2)]; end
% reset state variables and overflow counter
if(isempty(zs)), zs = zeros(nsec,2); end
noc = zeros(nsec,2);
xx = v;
for isec = 1:nsec
    if s(isec)~=1., 
        [xx,noc(isec,2)] = fixquant(xx * s(isec),w,'nearest',over);
    end
    b  = soc(isec,1:3);
    if(soc(isec,4)~=1), 
        error('FIXDF2TSOS: denominator coefficients?')
    end
    a = soc(isec,4:6);
    [xx,zs(isec,:),noc(isec,1)] = fx2tfi(b,a,xx,zs(isec,:),w,rmode,over);
end
xx = xx * s(nsec+1);
% set output parameter
y  = xx;
if (nargout>1), zss = zs; end
if (nargout>2),  nov = noc;
else
    if( max(noc(:))>0),
         warning('DSVLib:fixdf2tsos',' Overflow detected');
    end
end

%% Helper Functions----------------------------------------------
function [y,zi,noc]=fx2tfi(b,a,v,zi,w,rmode,over)
% 
xx = [zi 0]; aa=a(2:3);
noc = 0; y=zeros(size(v));
LEN = length(v); y=zeros(size(v));
n = 0;  
while n < LEN,
     n = n + 1;
     xx = v(n) * b + xx;
     [u,inoc] = fixquant(xx(1),w,rmode,over);
     noc = noc + inoc;
     y(n) = u;
     xx = xx(2:3);
     xx = xx - u*aa;
     [xx,inoc] = fixquant(xx,w,rmode,over);
     noc = noc+inoc;
     xx = [xx 0];
end
zi = xx(1:2);
%% [EOF] - fixdf2tsos

