function [X,noc] = fixquant( s,w,rmode,lmode )
%FIXQUANT: Quantization fixed-point arithmetic
%   Realization - Quantization modes for state variables
%       RMODE:  'convergent'    convergent rounding (mathematical
%                               rounding)
%               'nearest'       rounding to nearest level (two's comp)
%               'round','zero'  sign magnitude rounding   
%               'trunc','floor' 2's complement truncation
%               'magn' ,'fix'    magnitude truncation
%               'none'          no quantization
%   Realization - Overflow detection a correction mode 
%       LMODE:  'sat'           saturation limiter
%               'overfl' 'wrap' 2's complement overflow
%               'triang'        triangle limiter
%               'none'          no limiter
%   Default values:
%       round='round';    over='none'.
%
%   Syntax
%   X = fixquant( s,w)
%   X = fixquant( s,w,rmode)
%   X = fixquant( s,w,rmode,lmode )
%   [X,noc] = fixquant( s,w,...)
%
%   Description
% X = fixquant(s,w)returns the input signal s (scalar or 
%   array, real or complex) reduced to a word-length of 
%   w digits, including signum, and limited to the range [-1,1).
%   For the overflow characteristic over='sat' is used by default.
%   For complex s, real and imaginary part of s(k)are quantized and 
%   overflow checked seperately.
%
% X = fixquant( s,w,'convergent','sat' ) returns the input signal s to a 
%   word-length of w digits, quantized by convergent mode and overflow
%   correction by saturation characteristic.
%
% [X,noc] = fxquant(s,w,...) optionally returns
%   the number of occured overflows for active limiter check. nov=0 means
%   no overflow occured. 
%   If the output nov is not used, a warning message will be sent to the
%   workspace.
%
%   References: [DSV1, chap. 5.1 and 5.2]
%               [BuMcC94]
%
%   See also DSV-Lib: fixdf1sos, fixdf2sos
%            Matlab:  convergent, nearest, round, floor, fix

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Hans W. Schuessler and Guenter F. Dehner
%   Version: 1.0.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,4, nargin, 'struct'))
if nargin <3, rmode='round'; end
if nargin <4, lmode='none';  end
if w <= 0 || abs(rem(w,1)) > eps;
        error('FIXQUANT: wordlength must be positive integer');
end
if ~isinf(w)
%
Plus1 = 2^(w-1);
X = s * Plus1;
%
switch rmode(1:3)
    case 'con',        X = convergent(X);
    case 'nea',        X = nearest(X);
    case {'rou','zer'},X = round(X);
    case {'tru','flo'},X = floor(X);
    case {'mag','fix'},X = fix(X);
    case 'non',     % no quantization
    otherwise,   error('FIXQUANT: unknown wordlength reduction spec.');
end
%
nr=size(X,1);
if(~isreal(X)),
	flgcmplx =1;
	X = [real(X);imag(X)];
else
	flgcmplx =0;
end
switch lmode(1:3),
    case 'sat'
   Xs = min(Plus1 - 1,X);
   Xs = max(-Plus1,Xs);
%    if X~=Xs
%        error('FIXQUANT: Overflow')
%    end
   ovc=true;
    case {'ove','wra'}
   Xs = X + Plus1 * ( 1 - 2*floor((min(min(X),0))/2/Plus1) );
   Xs = rem(Xs,2*Plus1) - Plus1;
   ovc=true;
    case 'tri'
   Xs = X + Plus1 * ( 1 - 2*floor((min(min(X),0))/2/Plus1) );
   Xs = rem(Xs,4*Plus1) - Plus1;
   f = find(Xs > Plus1); Xs(f) = 2*Plus1 - Xs(f);
   f = find(X == Plus1); Xs(f) = Xs(f) - 1;
   ovc=true;
    case 'non'  %  limiter switched off
   Xs = X;
   ovc=false;
    otherwise,    error('FIXQUANT: unknown limiter spec.');
end
if ovc, f = find(X-Xs); noc = length(f); 
else    noc=0;
end
if (nargout<2 && noc ~=0), 
      warning('DSV_Lib:Overflow',' Overflow in FIXQUANT'); end
%
if(flgcmplx), Xs = complex(Xs(1:nr,:),Xs(nr+1:2*nr,:)); end
%
X = Xs / Plus1;
%
else
X=s;
end
%% [EOF] - fixquant
