function [y,zss,nov] = fixwf1sos(wdf,s,v,zs,w,rmode,over)
%FIXWF1SOS Quantized digital cascade filter with blocks in wf1 form
%   Realization of second order sections by wave digital filter blocks,
%   in the wf1 form, according to Meerkoetter and Wegener [MeeW76]
%
%   SYNTAX
%   y  = fixwf1sos(wdf,s,v)
%   y  = fixwf1sos(wdf,s,v,zs)
%   y  = fixwf1sos(wdf,s,v,zs,w)
%   y  = fixwf1sos(wdf,s,v,zs,w,rmode)
%   y  = fixwf1sos(wdf,s,v,zs,w,rmode,over)
%   [y,zss,nov]=fixwf1sos(wdf,s,v,...)
%
%   Description
%   ---------- in preparation ----------------
%                             
%   References: [DSV1, chap. 6.5.3.5]
%
%   See also DSV-Lib:  fixwf2sos, fixdf1sos, fixquant

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,7, nargin, 'struct'))
if (nargin<7), over = 'sat';  end
if (nargin<6), rmode='magn';end
if (nargin<5), w    = 16; end
if (nargin<4), zs = [];   end
nsec = size(wdf,1); ns=length(s);
if ns < 2, s=[s,ones(1,1)]; ns=2; end
if (ns < nsec+1), s = [s(1),ones(1,nsec+1-ns),s(2)]; end
% reset state variables and overflow counter
if(isempty(zs)), zs = zeros(nsec,2); end
noc = zeros(nsec,2);
xx = v;
%%
for isec = 1:nsec
  if s(isec)~=1., 
    [xx,noc(isec,2)] = fixquant(xx * s(isec),w,'nearest',over);
  end
  b  = wdf(isec,1:3);
  if(wdf(isec,4)~=1),  error('FIXWF1SOS: denominator coefficients?'); end
  a = wdf(isec,4:6);
  [xx,zs(isec,:),noc(isec,1)] = fxw1fi(b,a,xx,zs(isec,:),w,rmode,over);
end
xx = xx * s(nsec+1);
% set output parameter
y  = xx;
if (nargout>1), zss = zs; end
if (nargout>2),  nov = noc;
else
    if( max(noc(:))>0),
        warning('DSVLib:Overflow',' Overflow detected in FIXWF1SOS')
    end
end
%% Helper Functions ---------------------------------
function [y,zi,noc]=fxw1fi(b,a,v,zi,w,rmode,over)
% 
xx = [zi 0];   %x1,x2,0
b(2:3)=-b(2:3); 
noc = 0; y=zeros(size(v));
LEN = length(v); y=zeros(size(v));
n = 0;  
while n < LEN,
     n = n + 1;
     xx_ = v(n) * b + xx;
     xx_(1)= xx_(1)+xx(2);
     xx_(3)=xx_(3)-xx(1);
     [u,inoc] = fixquant(xx_(1),w,'nearest',over);
     noc = noc + inoc;
     y(n) = u;
    %y(n)=xx(2);
     xx_ = xx_ + u*a;
     [xx_,inoc] = fixquant(xx_(2:3),w,rmode,over);
     noc = noc+inoc;
     xx = [xx_ 0];
end
zi = xx(1:2);
%% [EOF] - fixf1sos
