function [y,zss] = flddf2sos(soc,s,v,zs,wm,rmode,over)
%FLDDF2SOS Float-point filter cascade realisation in df2sos form
%   Realisation
%   - summation of the products in double precission
%   - one quantizer after the summation point in the 2. order sections
%   - no extra quantizer after block scaling 
%   - quantization defined by rmode
%   - wordlength of mantissa wm (without sign)
%
% Floatpoint number overflow is currently not controlled!
%  
%   Default values:
%     wm = 16:   round='nearest'; 
%
%   Syntax
%   y = flddf2sos(sos,s,v)
%   y = flddf2sos(sos,s,v,zs)
%   y = flddf2sos(sos,s,v,zs,wm)
%   y = flddf2sos(sos,s,v,zs,wm,rmode)
%   [y ,zss]= flddf2sos(sos,s,v,...)
%
%   Description
%   ---------- in preparation ----------------
%                             
%   References: 
%
%   See also DSV-Lib:  flddf1, flddf1sos, fldquant

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,7, nargin, 'struct'))
%
if (nargin<7), over='none';    end
if (nargin<6), rmode='nearest';end
if (nargin<5), wm  = 16;       end
if (nargin<4), zs = [];        end
nsec = size(soc,1); ns=length(s);
if ns < 2, s=[s,ones(1,1)]; ns=2; end
if (ns < nsec+1), s = [s(1),ones(1,nsec+1-ns),s(2)]; end
% reset state variables
if(isempty(zs)), zs = zeros(nsec,2); end

%%
xx = v;
for isec = 1:nsec
    xx = xx * s(isec);   % no quantization after scaling mult.
    b  = soc(isec,1:3);
    if soc(isec,4)~=1, 
        error('FLDDF2SOS: denominator coefficients?')
    end
    a = soc(isec,4:6);
    [xx,zs(isec,:)] = fld2fi(b,a,xx,zs(isec,:),wm,rmode,over);
    % check for one or shift multipliers
    if(isec ~=nsec && ~ispower2(s(isec+1))), 
          xx = fldquant( xx, wm, rmode);
    end
    if isec==nsec,
       xx = fldquant( xx, wm,rmode);
    end
end
xx = xx * s(nsec+1);
% set output parameter
y  = xx;
if (nargout>1), zss = zs; end


%% Helper Functions----------------------------------------
function [y,zi] = fld2fi(b,a,v,zi,wm,rmode,over)
a = a(2:3);
x = zi.';
noc = 0.;
LEN = length(v);
n = 0;
y=zeros(size(v));
while n < LEN,
     n = n + 1;
     u = fldquant((v(n) - a * x),wm,rmode,over);
     y(n) = b*[u;x];
     x = [u;x(1)];
end
zi = x.';
%% [EOF] -flddf2sos
