function [y1,y2,zss] = fldfirlat(r,v,zs,wm,rmode)
%FLDFIRLAT FIR Float point filter in lattice strucure
%   Realisation
%   - summation of the products in double precission
%   - quantizer after the summation points 
%   - quantization defined by rmode
%   - wordlength of mantissa wm (without sign)
%  
%   Default values:
%     wm = 16:   rmode='nearest'; 
%
% Floatpoint number overflow is currently not controlled!
%
%   y1 upper exit;   y2 loer exit;
%
%   Syntax
%   [y1,y2] = fldfirlat(r,v)
%   [y1,y2] = fldfirlat(r,v,zs)
%   [y1,y2] = fldfirlat(r,v,zs,wm,rmode)
%   [y1,y2,zss] = fldfirlat(r,v,...)
%
%   Description
%   ---------- in preparation ----------------
%                             
% Example:
% phi=[0.6 0.3 -0.3 -0.6]';  r0 = 0.999;
% N=512;  v=zeros(1,2*N); v(1)=1;
%   z=r0*exp(j*phi*pi); 
%   b=zp2tf(z,1,1);
%  Reflection Factors
%   r=p2latc(b)
% [y,y1] = fldfirlat(r,v,[],16,'round');
%
%   References: [DSV1, chap. 6.7, p. 501]
%
%   See also DSV-Lib:  p2latc, fldiirlat, fldquant
%            Matlab:   tf2latc, latcfilt

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,5, nargin, 'struct'))
%
if nargin < 5, rmode='nearest'; end
if nargin < 4, wm=16;           end
if nargin < 3, zs=[];           end
%%
lr=length(r); lv = length(v); 
if isempty(zs), z=zeros(1,lr); else z=zs; end
y1=zeros(size(v)); y2=y1;
for k=1:lv
  a=v(k); b=a;
  for la=1:lr
    c=fldquant(z(la)+r(la)*b,wm,rmode);
    b=fldquant(z(la)*r(la)+b,wm,rmode);
    z(la)=a; a=c;
  end;
  y1(k)=b;      % upper exit
  y2(k)=a;      % lower exit
end
if nargout>2, zss=z; end
%% [EOF] - fldfirlat
