function [G,W,Hp]=blockfreqz(SSp,s,p,N,mode)
%BLOCKFREQZ Frequency response of block realization
%   The time invariant freqency response G and the shifted spectral parts Hp 
%   are calculated for the system matrix SSp.
%
%   Syntax
%   [G,W,Hp]=blockfreqz(SSp,s,p)
%   [G,W,Hp]=blockfreqz(SSp,s,p,N)
%   [G,W,Hp]=blockfreqz(SSp,s,p,N,mode)
%
%   SSp     systemmatrix
%   s       input and output scaling factors
%   p       blockfactor
%   N       number of frequency points
%   mode    extension of the frequency axes; 
%           'whole' N points around the entire unit circle
%
%   Description
%   ---------- in preparation ----------------
%
%   Example: DSV1, chap. 7.1.7.6
%                             
%   References: [DSV1, chap. 6.3]
%
%   See also DSV-Lib:  ss2block
%
%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,5, nargin, 'struct'))
if nargin<5 ||~strncmp(mode,'who',3), npi=1;
if nargin<4, N=512; end
else npi=2;
end
ns=size(SSp,1);
n=ns-p;
ss=prod(s);
Ap=SSp(1:n,1:n);    Bp=SSp(1:n,n+1:ns); 
Cp=SSp(n+1:ns,1:n); Dp=SSp(n+1:ns,n+1:ns);
%
W=(0:N-1)'/N*npi*pi;   z=exp(j*W); zp=z.^p;
Hp=zeros(N,p,p);
G=zeros(N,p);
pp = 0:p-1;
wp  = exp(-j*2*pi/p);  wpp = wp.^pp;
for i=1:N,
    % calculate transfer matrix Hp(zp)
    HH = ss * ( Cp * (zp(i)*eye(n)-Ap)^-1 * Bp + Dp);
    Hp(i,:,:)=HH;
    % calculate time invariant transfer function
    for k=1:p,
    G(i,k)=(z(i).^-pp * HH* (z(i).^pp .* wpp.^(k-1)).')/p;
    end
end
%% [EOF] - blockfreqz


