function [Hp,W] = parfreqz(sop,sp,varargin)
%SOPFREQZ  Frequency response of Parallel Form
%   The frequency response of the parallel arranged sections is calculated 
%   and summed up.
%
%   Syntax
%   [Hp,W] = parfreqz(sop)           w/o scaling factors
%   [Hp,W] = parfreqz(sop,sp)        default L=512
%   [Hp,W] = parfreqz(sop,sp,L)
%   [Hp,W] = parfreqz(sop,sp,f,fs)   for further input parameters 
%                                    see Matlab: freqz
%
%  sop,sp   Set of SOP coefficients and scaling factors
%  L        Number of normalized frequency points
%  f, fs    Frequency Response at frequencies f and sampling frequency fs
%
%
%   References: [DSV1, chap. 5.2]
%
%   See also DSV-Lib: sosfreqz
%            Matlab:  freqz

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Author: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by author(s) - not released for commercial use
%%  ---------------------------------------------------------------
if nargin<2, sp=[]; end
if nargin<3,
     argin={512};
else argin=varargin;
end
%
nb = size(sop,1)-1;
[s1,s2,s3]=parse_sp(sp,nb);
%
hp=0.;
for i=1:nb
 [hpp,w]= freqz(sop(i,1:3),sop(i,4:6),argin{:});
 hp = hp + (s2(i))*hpp;
end
hp = s1*s3*(hp +sop(nb+1,1));
%% 
if nargout==0,
    plot(w/pi,h2db(hp),'b')
    xlabel('Normalized Frequency \Omega/\pi'); ylabel('Magnitude (dB)')
    grid on
else
    Hp = hp;
    if(nargout>1), W=w; end
end
%% Helper Function
function [s1,s2,s3]=parse_sp(sp,nb)
if ~isempty(sp)
    terr='PARFREQZ: mismatched scaling parameters';
  [Lsp,Csp]=size(sp)
  switch Csp,
  case 3
    if Lsp~=nb, error(terr); end
    s1=sp(1,1); s2=sp(1:nb,2); s3=sp(1,3);
  case 2
    if Lsp~=1, error(terr); end
    s1=sp(1,1); s2=ones(nb,1); s3=sp(1,2);
  case 1
    if Lsp~=nb, error(terr); end
    s1=1; s2=sp; s3=1;
  otherwise,    error(terr)
  end
else
    s1=1; s2=ones(nb,1); s3=1;
end
%% [EOF] - parfreqz
