function [H,W] = sosfreqz (sos,s,varargin)
% SOSFREQZ - Frequency Response for SOS coefficient input
%   The frequency response is directly calculated from the SOS coefficients 
%   without transformation into the direct form.
%
%   Syntax
%   [H,W] = sosfreqz(sos,s)          default L=512
%   [H,W] = sosfreqz(sos,s,L)
%   [H,W] = sosfreqz(sos,s,f,fs)     or further input parameters 
%                                    see Matlab: freqz
%
%  sos,s    Set of SOS Coefficients
%  L        Number of normalized frequency points
%  f, fs    Frequency Response at frequencies f and sampling frequency fs
%
%
%   References: [DSV1, chap. 5.2, 5.4, 7.1.6.6]
%
%   See also DSV-Lib: parfreqz
%            Matlab:  freqz

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Author: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by author(s) - not released for commercial use
%%  ---------------------------------------------------------------
[h,w]=freqz(sos(1,1:3),sos(1,4:6),varargin{:});
h=prod(s)*h;
nb=size(sos,1);
for i = 2:nb
    h = h.*freqz(sos(i,1:3),sos(i,4:6),varargin{:});
end
%%
if nargout==0,
    plot(w/pi,h2db(h),'b')
    xlabel('Normalized Frequency \Omega/\pi'); ylabel('Magnitude (dB)')
    grid on
else
    H = h;
    if(nargout>1), W=w; end
end
%% [EOF] - sosfreqz
   
