function axarrowline(X,Y,varargin)
% AXARROWLINE Line plot with additional arrow at the end
%   Line is drawn in actual axes coordinates
%
%   Syntax
%   axarrowline(X,Y)
%   axarrowline(X,Y,...)
%
%   X   vector containing x-coordinates of the line
%   Y   vector containing y-coordinates of the line
%   ... additional parameters in accordance with Matlab function line(.)
%
% To resize the arrow graphs after zooming the axes or resizing the figure
% window the callback functions 'ActionPostCallback' and 'ReziseFcn' are
% installed. This may cause a conflict using this mechanism elsewhere in
% the program.
%
% Remark: AXARROWLINE draws vectors or lines in the current AXES space 
% ending by an arrow.  The function has to be distinct from the 
% MATLAB internal helper function ARROWLINE working in the FIGURE space.
%
%   References: [DSV1: chap. 7.1.7.1]
%
%   See also Matlab:  line

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
hl=line(X,Y,varargin{:});
ha=gca;
arrow (ha,hl,[X(end-1) X(end)],[Y(end-1) Y(end)])
h = zoom;
set(h,'ActionPostCallback',@dsvpostcallback);
set(h,'Enable','off');
set(gcf,'ResizeFcn',@dsvresizeFcn);
%%
% --- helper function -----
function arrow (ha,hd,X,Y)
if nargin==2, 
    recon=true; 
    STRU=get(hd,'UserData');
    X=STRU.Data(:,1); Y=STRU.Data(:,2);
    hp=STRU.LineParent;

else
    recon=false;  % build new arrow symbol
    hp=hd;
end
%
% size of the symbol
msize=get(hp,'MarkerSize');
mecol=get(hp,'Color');
mfcol=get(hp,'MarkerFaceColor');
if isequal(mfcol,'auto')|| isequal(mfcol,'none'), 
    mfcol=mecol; end
sw = 4*msize;                  % defined by MarkerSize
lw =   get(hp,'LineWidth');    % defined by LineWidth
if iscell(lw), lw=lw{end}; end
sw = max(sw * lw/2, 6);        % set minimal size
% get scaling of the symbol
uno=get(gca,'Units');  
set(gca,'Units','Points');
pos=get(ha,'Position');
set(ha,'Units',uno);
xlim=get(ha,'XLim'); ylim=get(ha,'YLim');
width=pos(3); hight=pos(4);
if isequal(get(ha,'PlotBoxAspectRatioMode'),'manual')
    rat=get(ha,'PlotBoxAspectRatio');
    if hight<width,
        width=hight*rat(1)/rat(2);
    else
        hight=width*rat(2)/rat(1);
    end
end
vscale = -diff(ylim)/hight; hscale = -diff(xlim)/width;
% get angle (rotation par) of the symbol
x1=X(end-1); x2=X(end);  y1=Y(end-1); y2=Y(end);
x1=(x2-x1)/hscale;       y1=(y2-y1)/vscale;
hy=sqrt(x1*x1+y1*y1);
co=x1/hy;                si=y1/hy;
% build symbol and rotate
sh=0.66*sw;  % symbol hight in relation to width
xt=[0,-sw,-sw,0];                % symbol layout
yt=[0,sh/2,-sh/2,0];
obm =[co -si; si co] * [xt; yt]; % rotate symbol
xt=obm(1,:) * hscale + x2;       % rotated coordinates
yt=obm(2,:) * vscale + y2;
if recon 
  set(hd,'XData', xt,'YData',yt)
else
  hp=patch(xt,yt,'k','EdgeColor',mecol,'FaceColor',mfcol);
  stru=struct('LineParent',hd,'Data',[X(:),Y(:)]);
  set(hp,'Tag','dsv_arrowID',...
             'UserData',stru)
end

%% Helper Function
function dsvpostcallback(obj,evd)
comp = findall(evd.Axes,'Tag','dsv_arrowID');
nc=length(comp);
for n=1:nc,
    arrow(evd.Axes,comp(n))
end

function dsvresizeFcn(src,evt)
hfig=gcbo;
comp = findall(hfig,'Tag','dsv_arrowID');
nc=length(comp);
for n=1:nc,
    ha=get(comp(n),'Parent');
    arrow(ha,comp(n))
end
%% EOF - axarrowline