function hd=plot_bifreq(mode,x,y,z,varargin)
%PLOT_BIFREQ Plot bi-frequencial transfer functions in 3D
%
%   Syntax
%   hd=plot_bifreq(mode,x,y,z)
%   hd=plot_bifreq(mode,x,y,z,...)
%
%   mode    control parameter for subfunctions
%   1. step plot axes
%       hd=plot_bifreq('axes',x,y,ZLim)
%   following steps plot functions in different appearance
%       hd=plot_bifreq('patch',x,y,z,CData)
%   or
%       hd=plot_bifreq('hatch',x,y,z,CData,Marker)
%
%   Entire Description
%   ---------- in preparation ----------------
%
%   For more information use example: DSV1, chap. 7.1.7.6
%                             
%   References: [DSV1, chap. 7.1.7.6]
%
%   See also DSV-Lib:  

%% ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------

switch mode
  case 'axes'
    if nargin<2 || isempty(y), y=x; end
    if nargin<3,z=[-100,0];      end
    [X,Y]=meshgrid(x,y);
    [Nx,Ny]=size(X);
    Z=z(1)*ones(Ny,Nx);
    hd=mesh(X,Y,Z);
    set(gca,'ZLim',z,'View',[-17 38]);
    xlabel('\Omega_1 /\pi')
    ylabel('\Omega_2 /\pi')
    set(gcf,'UserData',[Nx-1,Ny-1]);
    %
  otherwise
    func=['bi' mode]; p=size(z,2);
    cdat=[];
    if nargin>4, cdat=varargin{1}; end
    if ~isempty(cdat),
        if length(cdat)==1,
            col=colormap; Ncol=size(col,1);
            if cdat<0 || cdat>1, 
              error('plot_bifreq: indexed color out of range');
            end
            ind= round(cdat*Ncol);
            cdat=col(ind,:);
        end
    else
      col=colormap; Ncol=size(col,1);
         if cdat<0 || cdat>1, 
           error('plot_bifreq: indexed color out of range');
         end 
         ind=round((0:1:p-1)/(p-1)*(Ncol-1))+1;
         cdat=col(ind,:);
    end
    switch mode
        case 'patch'
            lord=['- ';': ';'--';'-.'];
        case 'hatch'
            lord=['+','.','o','s']';
    end
    zlim=get(gca,'ZLim');    % definition of basis line
    z(z<=zlim(1))=zlim(1)+1;
    ylim=get(gca,'YLim');    % wrap in Omega_2 direction
    ymin=ylim(1); ymax=ylim(2);
    n=size(x,2);
    for i=1:n,
        ii=rem(i,size(lord,1));
        if ii==0, ii=size(lord,1); end
    int=find(y(:,i)>=ymin & y(:,i)<=ymax);
    x1=x(int,i);  y1=y(int,i); z1=z(int,i);
    hd = feval(func,x1,y1,z1,cdat(i,:),lord(ii,:));
    x1=x(:,i); x1(int)=[];
    if ~isempty(x1),
      y1=y(:,i); y1(int)=[]; y1=y1+ymax;
      z1=z(:,i); z1(int)=[]; 
      hd = feval(func,x1,y1,z1,cdat(i,:),lord(i,:));
    end
    end
end
%% -------------------------------------------------------------
function hd=bipatch (x,y,z,col,lst)
  X = [x(:); x(end); x(1); x(1)];
  Y = [y(:); y(end); y(1); y(1)];    
  Z = [z(:); -100; -100; z(1)];
  hd=patch(X,Y,Z,col,'EdgeColor','none');
  line(x(end-3:end),y(end-3:end),z(end-3:end),...
      'Color','k','Linewidth',0.5)
  line(x,y,z,'Color','k','Linewidth',1.5,'LineStyle',lst)
%
function hd=bihatch (x,y,z,varargin)
  X = [x; x(end); x(1); x(1)];
  Y = [y ;y(end); y(1); y(1)];    
  Z = [z; -100; -100; z(1)];
  hd=patch(X,Y,Z,[1 1 1]);
  Nxy=get(gcf,'UserData');
  xlim=get(gca,'XLim'); Dx=xlim(2)/Nxy(1);  
  NN=length(X)-3;
  xs=X(1); xe=X(end-2); 
  ys=Y(1); ye=Y(end-2);
  dx = (xe-xs)/NN; dn = floor(Dx/dx); 
  Dx = dn*dx;
  dy = (ye-ys)/NN;
  Dy = dn*dy;
  zlim=get(gca,'ZLim'); z0=zlim(1);
  nn=0; xx=xs; yy=ys;
  dz=10;
  doff=dz/4;
  while xx < xe-Dx;
    nn=nn+dn;  
    xx= xx+Dx;  yy=yy+Dy;  zz=Z(nn);
    nz=floor((zz-z0)/dz);
    XS=xx*ones(1,nz);
    YS=yy*ones(1,nz);
    ZS=(1:1:nz)*dz+z0+doff;  doff=-doff;
    ZS=min(ZS,zz-0.5*dz);
    mar='+';
    if ~isempty(varargin{2}), mar=varargin{2}; end
  line(XS,YS,ZS,'Marker',mar,'LineStyle','none',...
        'MarkerEdgeColor',[0.8 0.8 0.8]);
  end
%% EOF - plot_bifreq