function [H,W,Pnn,Ri] = fixnlm(varargin)
%FIXNLM Noise loaded method to estimate frequency response                  
%  and power density sectrum of quantization errors
%
%   Syntax
%   [H,W] = fixnlm('filtername',{coeff},{addFiltPar})
%   [H,W] = fixnlm('filtername',{coeff},{addFiltPar},N,L)
%   [H,W] = fixnlm('filtername',{coeff},{addFiltPar},N,L,we)
%   [H,W,Pnn,Ri] = fixnlm(...)
%
%   Description
% [H,W] = fixnlm('filtername',{coeff},{addFiltPar}) measures the estimate 
%   of the transfer function for the fixed-point filter, defined 'filtername'
%   the coefficients specified in {coef} and the parameters for control of
%   quantization and overflow, defined in the cell {addFiltPar}). N and L 
%   are chosen by default. The frequency points are returned in vector W.
% [H,W] = fixnlm(...,N,L)defines in addition the number of frequency 
%   points N and the number of trails L.
% [H,W] = fixnlm(...,we)controls an additional quantization of the 
%   test signal at the filter entrance to we digits.
% [H,W,Pnn,Ri] = fixnlm(...) returns the power density function Pnn of the 
%   uncorrelated error and the noise figure Ri.
%
% Example:
% Measurement of 2nd canonical form by the filter module 'fixdf2sos',
%    
%  [H,W,Pnn,Ri] = fixnlm('fixdf2sos',{sos,s},{ws,rmode,over},N,L,we)
%  Filter function    [y,zs,Nov] = sosfx2filter (soc,s,v,z,ws,rmode,over)
%   soc     SOS Coefficient Block
%   s       ScaleValues vector
%   v       Input signal
%   z       Intitial State Variables; z=[]: initial values set to z(i,j)=0
%   ws      internal word length
%   rmode   quantization mode
%   over    overflow mode
%
%   References: [DSV1, chap. 7.1.7.6]
%               [SchD90], [BuMcC94]
%
%   See also DSV-Lib: fixdsf2sos, fixquant
%            Matlab:  feval, waitbar

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Hans W. Schuessler and Guenter F. Dehner
%   Version: 1.0.2
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,6, nargin, 'struct'))
%
if ~ischar(varargin{1}), error('nlm: filtname is not a string'); end
if ~iscell(varargin(2)), error('nlm: coeff is not a cell array'); end
if ~iscell(varargin(3)), error('nlm: addfiltPar is not a cell array'); end
%
filtername=varargin{1};
varg1={varargin{2}{:}}; varg2={varargin{3}{:}};
if nargin<6, we=[]; else we=varargin{6}; end
if nargin<5, L=20;  else L =varargin{5}; end
if nargin<4, N=512; else N =varargin{4}; end
z=[]; 
wi=varg2{1};
%
N = 2*N;
sumH = zeros(1,N);
sumY2 = sumH;
%
hd = waitbar(0,'NLM Simulation  -- Pusch Cancle to Stop ','CreateCancelBTN','closereq'); 
set(hd,'Position',[100 100 260 60]);
drawnow;
%% mean over L measurements
for i = 1:L,
    phi1 = 2*pi*rand(1,N/2-1);
    phi = [0 phi1 0 -phi1(N/2-1:-1:1)];
    Vp = exp(1i*phi);     % |Vp| = 1
    vp = real(ifft(Vp));                       
    if(~isempty(we)),                                 
        vp = fixquant( vp, we, 'round', 'none' );   
    end
    v  = [vp, vp];
    y = feval(filtername,varg1{:},v,z,varg2{:});
     y = y(N+1:1:2*N);
     Yp = fft(y);

     sumH = sumH + Yp./Vp;
     sumY2 = sumY2 + real(Yp.*conj(Yp));
    %
     try
     waitbar (i/L,hd); drawnow;
     catch                       %#ok<CTCH>
     H=[];W=[];Pnn=[]; Ri=[];
     warn=warning('off','backtrace');
     warning(['DSVLib:interCancel',' Measurement Stopped at L = ',int2str(i)])
     warning(warn);
     return
     end
end
close (hd);
%    
%% Evaluation of the results
Hx = sumH/L;
if(L==1), error ('SOSNLM: L has to be greater than 1'); end
Px = abs(sumY2/(L-1)- real(Hx.*conj(Hx))*L/(L-1))/N ;
                     % Correction:  scaling by (L-1)
Q = 2^(1-wi);

Rix = mean(Px) / (Q^2/12);
N = N/2;
Wx = (0:1/N:1-1/N);
Hx = Hx(1:N);
Px = Px(1:N);
if (nargout >0),
    H   = Hx;
    W   = pi*Wx;
    if nargout>2, Pnn = Px;  end
    if nargout>3, Ri  = Rix; end
else
   figure;
    plot (Wx,p2db(Px));
    hold on
    plot (Wx, h2db(Hx),'r')
    hold off
    legend ('Noise Spectrum', 'Transfer function')
    text(0.1,0.95,['Noise Figure Ri: ' num2str(Rix)],'Units','normalized')
    set (gca,'FontSize',8)
    grid on
end
%% [EOF] - fixnlm


