function [Pxx,W] = wblds(v,n0,dn,M,ft)
%WBLDS  Power spectrum density according to "Welch-Barlett"
%   German: Leistungsdichtespectrum (LDS)
%
%   Syntax
%   [Pxx,W] = wblds(v,n0,dn,M,ft)
%
%   Description
% [Pxx,W] = wblds(v,n0,dn,M,ft) calculates the PSD for a sequence v,
%   partitioned into seguments of length n0 and a shift of dn samples.
%   M defines the length of the FFT. Windowing is controlled by the logical
%   parameter ft.
%   ft=true    Barlett Window (default: no windowing)
%
%   References: [DSV1, chap. 3.3.5.2]
%
%   See also DSV-Lib: covlds
%            Matlab:  fft

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Hans W. Schuessler
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,5, nargin, 'struct'))
if nargin<5, ft=false; end
if nargin<4, M=1024; end
% N Number of segments
n = length(v); N = 1+floor((n-n0)/(n0-dn));
v = v(:)';
if ft,
    f = sqrt(2/3)*(1-cos(2*pi*[0:n0-1]/n0)); 
else
    f = ones(1,n0);
end
%% Mean over N segmented periodograms
lds = zeros(1,M/2);
for i=0:N-1,
   ind = i*(n0-dn);
   perseg = fft(v(ind+1:ind+n0).*f,M);
   perseg = perseg.*conj(perseg)/n0;
   lds = lds + perseg(1:M/2);
end
Pxx = (lds/N)';
if nargout>1,W = (0:M/2-1)*2*pi/M; end
%% [EOF] - wblds
