function [sos_,s_]=scaling_df1sos(sos,s,norm,over)
%SCALING_DF1SOS Scaling of SOS cascade with blocks in the df1-form
%        Scaled structure without separate block scaling factors
%
%   Syntax
%   [sos_,s_]=scaling_df1sos(sos,s)
%   [sos_,s_]=scaling_df1sos(sos,s,norm)
%   [sos_,s_]=scaling_df1sos(sos,s,norm,over)
%
%   Description
% [sos_,s_] = scaling_df1sos(sos,s,'abs') The filter defined by the 
%   SOS block and the scaling parameters s is scaled for the df1sos-form 
%   according to the absolute norm (norm = 'abs'). 
%   The df1-form is realized using combined SOS blocks. Quantization is
%   carried out after summation of all products [Deh03]. 
%
% Parameters:
%   sos     SOS coefficients
%   s       block scaling factor(s)
%   norm    Scaling Norm    (default: 'Linf')
%           absolute norm:  'l1' or 'abs'
%           L-inf-norm:     'Linf' or 'L-inf'
%           L-2-norm:       'L2' or 'L-2'
%   over    Overflow characteristic (default: 'sat')
%           Saturation:         'sat' or 'saturate'
%           Two's Complement:   'wrap'
% Results:
%   sos_    Scaled SOS Coefficients 
%   s_      block scaling factors, set to 1. 
%
%   References: [DSV1, chap. 5.2], [Deh03]
%
%   See also DSV-Lib: fixdsf1sos, dsv_filternorm
%            Matlab:  conv

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors:  Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for comercial use
%%  ---------------------------------------------------------------
error(nargchk(2,4, nargin, 'struct'))
%
if nargin<4, over='sat';  end
if nargin<3, norm ='inf'; end
%
nb = size(sos,1);
scalm = 1.;
s_=ones(nb+1,1);  sos_=sos;
%
bb = 1.; aa = 1.; sp=1.;
for i = 1:nb
  bb1 = conv (bb,sos(i,1:3));
  aa = conv (aa,sos(i,4:6));
  fact = 1/dsv_filternorm(bb1,aa,norm);
  switch over
    case {'wrap','sat','saturate'}       % no additional scaling points to check
    otherwise,error('SCALING_DF1SOS -- parameter not def.');
  end
  bb=bb1;
  if (fact > scalm), fact = scalm; end
  sp =sp * fact;  bb = bb * fact;
  sos_ (i,1:3) = sos(i,1:3) * fact;
end
%% set final scaling factor
s_(nb+1) = prod(s) / sp;
%% [EOF] - scaling_df2tsos
