function [sos_,s_]=scaling_df2sos(sos,s,norm,over)
%SCALING_DF2SOS Scaling of SOS cascade with blocks in the df2-form
%        Scaled structure without separate block scaling factors
%
%   Syntax
%   [sos_,s_]=scaling_df2sos(sos,s)
%   [sos_,s_]=scaling_df2sos(sos,s,norm)
%   [sos_,s_]=scaling_df2sos(sos,s,norm,over)
%
%   Description
% [sos_,s_] = scaling_df2sos(sos,s,'abs') The filter defined by the 
%   SOS block and the scaling parameters s is scaled for the df2sos-form 
%   according to the absolute norm (norm = 'abs'). 
%   Scaling points are currently tested in front of the state variables, 
%   according to overflow modes 'wrap' and 'sat'('saturate').
%
% Parameters:
%   sos     SOS coefficients
%   s       block scaling factor(s)
%   norm    Scaling Norm    (default: 'Linf')
%           absolute norm:  '1','l-imp','abs'
%           L-inf-norm:     'Linf' or 'L-inf'
%           L-2-norm:       'L2' or 'L-2'
%   over    Overflow characteristic (default: 'sat')
% Results:
%   sos_    Scaled SOS Coefficients 
%   s_      block scaling factors, set to 1.
%
%   References: [DSV1, chap. 5.2], [Deh03]
%
%   See also DSV-Lib: fixdsf2sos, dsv_filternorm
%            Matlab:  conv

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,4, nargin, 'struct'))
if nargin<4, over ='sat'; end
if nargin<3, norm ='inf'; end
%
nb = size(sos,1);
scalm = 1.;
s_=ones(nb+1,1);  
%
switch over
case {'wrap','sat','saturate'}
%% Scaling in front of the first block
    bb =1;
    aa = sos(1,4:6);
    fact =1/dsv_filternorm(bb,aa,norm);
     if (fact > scalm), fact = scalm; end 
    bb   = fact;
    s_(1) = fact;
%% Scaling within the cascade
    for i = 1:nb-1
        bb = conv (bb,sos(i,1:3));
        aa = conv (aa,sos(i+1,4:6));
        fact = 1/dsv_filternorm(bb,aa,norm);
         if (fact > scalm), fact = scalm; end 
        bb = bb * fact;
        s_(i+1) = fact;
    end
%% Scaling of the final block
      bb = conv (bb,sos(nb,1:3));
      fact = 1/dsv_filternorm(bb,aa,norm);
      if (fact > scalm), fact = scalm; end 
      s_(nb+1) = fact;
%% Multiplication of numerators by the scaling factors
      sos_=sos;
      sp = prod(s_);
      for i = 1:nb
         sos_(i,1:3) = sos(i,1:3) * s_(i+1);
         s_(i+1) = 1.;
      end
otherwise,error('SALING_DF2SOS: parameter not def.');
end
%% Set final scaling factor
s_(nb+1) = prod(s) / sp;
%% [EOF] - scaling_df2sos
