function [sss,s] = scaling_ssfsos(sss,g,optstr,norm)
%SCALING_SSFSOS Scaling of the SOS cascade in state space form
%
%   Syntax
%   [sos_,s_]=scaling_ssfsos(sss)
%   [sos_,s_]=scaling_ssfsos(sss,s)
%   [sos_,s_]=scaling_ssfsos(sss,s,optstr)
%   [sos_,s_]=scaling_ssfsos(sss,s,optstr,norm)
%   [sos_,s_]=scaling_ssfsos(sss,s,optstr,norm,over)
%
%   Description
% sss = scaling_ssfsos(sss,g,optstr,norm)
%
%   SSS     Cell Matrix for SOS State Space Object 
%   g       additional input scaling factor (default = 1.)
%   cstr    scaling for type optstr: 'normal' 'optimal'(default)
%   norm    scaling by chosen norm: 'abs' 'inf' 'two'(default)
%
%   References: [DSV1, chap. 6.3, 7.1.7.6], [Deh03], [Jac96]
%
%   See also DSV-Lib: sos2ssf, dsv_filternorm
%            Matlab:  conv

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,4, nargin, 'struct'))
if(nargin<4), norm='two'; end
if(nargin<3), optstr='optimal'; end
if(nargin<2), g = 1.; end
%
nb = size(sss,1);
bb = 1;  aa = 1;  dd = 1; d0=g; s=ones(1,2);
tol = 1.e-3;  % nlm=100;
for ib = 1:nb,
  [A,B,Ct,D]=sss_getblk(sss,ib);  % Block ib
  n=size(A,1);
  if n==1,                % Direct Scaling no optimization
     [b,a] = ss2tf(A,B,1,0,1); 
     b1 = conv(bb,b);  aa = conv(aa,a);
     Fn1 = dsp_filternorm (b1,aa,norm);
     B = B/Fn1;  Ct=Ct*Fn1;
  else
   [b,a] = ss2tf(A,B,[1 0],0,1); aa = conv(aa,a);
  switch optstr
  case 'normal'
    switch norm
    case {2 'two' 'L-2' 'L-two'}
      [b1,a] = ss2tf(A,B,[1 0],0,1);          %#ok<NASGU>
      b1 = conv(bb,b1);   Fn1 = dsv_filternorm (b1,aa,norm);
      [b2,a] = ss2tf(A,B,[0 1],0,1);          %#ok<NASGU>
      b2 = conv(bb,b2);   Fn2 = dsv_filternorm (b2,aa,norm); 
      csn = crosspectnorm(b1,aa,b2,aa);
      if(abs(csn)>eps),
        theta = atan2(Fn2^2 - Fn1^2,2*csn)/2;
      else
        theta =0.;
      end
    %
    case {1 inf 'inf' 'L-inf' 'abs'}  % Optimization {Fn2-Fn1}=0 !
      options = optimset('Display','notify','TolX',tol);
      [theta,fval,exitflg]=fzero(@optfun,0.,options,A,B,bb,aa,norm);
      if exitflg<0, error('SSF_SCALING: No convergence for Normal Mode'); end 
    end
    T = [cos(theta) sin(theta); -sin(theta) cos(theta)];
    B1 = T*B;
    [b1,a] = ss2tf(A,B1,[1 0],0,1);        %#ok<NASGU>
    b1 = conv(bb,b1);   Fn1 = dsv_filternorm (b1,aa,norm);
    T = 1/Fn1*T;   
    A = T*A*T^-1;  B = T*B;   Ct= Ct*T^-1;    
    %      
  case 'optimal'
    [b1,a] = ss2tf(A,B,[1 0],0,1);          %#ok<NASGU>
    b1 = conv(bb,b1);   Fn1 = dsv_filternorm (b1,aa,norm);
    [b2,a] = ss2tf(A,B,[0 1],0,1);          %#ok<NASGU>
    b2 = conv(bb,b2);   Fn2 = dsv_filternorm (b2,aa,norm); 
    T = [1/Fn1 0; 0 1/Fn2];
    A = T*A*T^-1;  B = T*B;   Ct= Ct*T^-1; 
  otherwise, error ('SSS_SCALING: not allowed structure type');
  end
  end
  [b3,a] = ss2tf(A,B,Ct,D,1);  d0 = d0*b3(1);      %#ok<NASGU>
  bb = conv(bb,b3);   Fn3 = dsv_filternorm(bb,aa,norm);
  D = D/Fn3; Ct= Ct/Fn3;
  dd = dd*D; bb = bb/Fn3;
  sss(ib,:)=[{A} {B} {Ct} {D}];
end
s(2) = d0/dd;
%%  Helper function -------------------------------------------------------
function fx=optfun(theta,A,B,bb,aa,norm)
    T = [cos(theta) sin(theta); -sin(theta) cos(theta)];
    B1 = T*B;
    [b1,a] = ss2tf(A,B1,[1 0],0,1);        %#ok<NASGU>
    b1 = conv(bb,b1);   Fn1 = dsv_filternorm (b1,aa,norm);
    [b2,a] = ss2tf(A,B1,[0 1],0,1);        %#ok<NASGU>
    b2 = conv(bb,b2);   Fn2 = dsv_filternorm (b2,aa,norm);
    %
    fx = Fn2-Fn1;
%% [EOF] - scaling_ssfsos

