function [wdf_,g_]=scaling_wf2sos(wdf,g,norm,over)
%SCALING_WF2SOS Scaling of cascade with blocks in the wf2-form
%        Scaled structure without separate block scaling factors
%
%   Syntax
%   [wdf_,g_]=scaling_wf2sos(wdf,g)
%   [wdf_,g_]=scaling_wf2sos(wdf,g,norm)
%   [wdf_,g_]=scaling_wf2sos(wdf,g,norm,over)
%
%   Description
% [wdf_,g_] = scaling_wf2sos(wdf,g,'abs') The filter defined by the 
%   SOS block and the scaling parameters s is scaled for the wf2sos-form 
%   according to the absolute norm (norm = 'abs'). 
%   Scaling points are currently tested in front of the state variables, 
%   according to overflow modes 'wrap' and 'sat'('saturate').
%
% Parameters:
%   wdf     SOS coefficients for wave digital filters
%   g       block scaling factor(s)
%   norm    Scaling Norm    (default: 'Linf')
%           absolute norm:  'l1' or 'abs'
%           L-inf-norm:     'Linf' or 'L-inf'
%           L-2-norm:       'L2' or 'L-2'
%   over    Overflow characteristic (default: 'sat')
% Results:
%   wdf_    Scaled SOS Coefficients 
%   g_      block scaling factors, set to 1. 
%
%   References: [DSV1, chap. 6.6.3.5]
%
%   See also DSV-Lib: sos2wdf, dsv_filternorm
%            Matlab:  conv

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,4, nargin, 'struct'))
if nargin<4, over ='sat'; end
if nargin<3, norm ='inf'; end
%
nb = size(wdf,1);
sos=wdf2sos(wdf,1);
scalm = 1.;
g_=ones(nb+1,1);  
bbs=[1 1 0;1 -1 0; 1 0 1];
%
switch over
case {'wrap','sat','saturate'}
%% Scaling in front of the first block
    aa = sos(1,4:6);
    fact=scalm;
    for k=1:3
      factk =1/dsv_filternorm(bbs(k,:),aa,norm);  
      fact=min(fact,min(factk,scalm));
    end
    bb   = fact;
    g_(1) = fact;
%% Scaling within the cascade
    for i = 1:nb-1
        bb = conv (bb,sos(i,1:3));
        aa = conv (aa,sos(i+1,4:6));
        fact=scalm;
        for k=1:3
          bbb=conv(bb,bbs(k,:));
          factk =1/dsv_filternorm(bbb,aa,norm);
         % 1/factk
    %figure(k); [H1,W]=freqz(bbb,aa); plot(W/pi,h2db(H1)); grid on;
          fact=min(fact,min(factk,scalm));
        end
        bb = bb * fact;
        g_(i+1) = fact;
    end
%% Scaling of the final block
      bb = conv (bb,sos(nb,1:3));
      fact = 1/dsv_filternorm(bb,aa,norm);
      fact = min(fact,scalm);
      g_(nb+1) = fact;
%% Multiplication of numerators by the scaling factors
      wdf_=wdf;
      gp = prod(g_);
      for i = 1:nb
         wdf_(i,1:3) = wdf(i,1:3) * g_(i+1);
         g_(i+1) = 1.;
      end
otherwise,error('SALING_DF2SOS: parameter not def.');
end
%% Set final scaling factor
g_(nb+1) = prod(g) / gp;
%% [EOF] - scaling_wf2sos
