function [v,t]=sinewave(a,f,phi,t,fa) 
%SINEWAVE Multifrequent sinusoidal signal
%
%   Syntax
%   [v,t]=sinewave(a,om,phi,t) 
%   [v,t]=sinewave(a,f,phi,t,fa) 
%
%   Description 
% v=sinewave(a,om,phi,t) builds a multifrequent signal, using amplitudes
%   in vector a, normalized frequencies in vector om and startphases in vector phi. 
%   In the case, that t is a vector, the signal is calculated for all values
%   of t. Otherwise t is a skalar (integer) and the signal is calculated 
%   for K=t time steps of delta_t=1. If parameter phi is a skalar, all
%   waves start with the same phase phi.
% v=sinewave(a,f,phi,t,fa) builds the signal for all frequencies in vector f.
%   The parameter fa defines the sampling frequency.
% [v,t]=sinewave(...) calculates the multifrequent signal and optionally  
%   outputs the time vector t

%   References: [DSV1, Chap. 2.4.2 ]
%
%   See also Matlab: repmat, sin

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(4,5, nargin, 'struct'))
%
if nargin<5,  om=f;
else          om=2*f/fa;
end
K=length(t);
if K==1, 
    K=t; 
    t= (0:1:K-1)';
end
lenf=length(f);
if length(phi)~=lenf,
    if length(phi)~=1, error('SINEWAVE: Phases?'); end
    phi=phi*ones(1,lenf);
end
if length(a)~=lenf,
    if length(a)~=1, error('SINEWAVE: Amplitudes?'); end
    a=a*ones(1,lenf);
end
tp=repmat(phi,K,1);
aa=repmat(a,K,1);
v=aa.*sin((t*om+tp)*pi);
v=sum(v,2);
%% [EOF] - sinewave
