function psi = rader(v1,v2,L)
%RADER Calculation of the auto- and crosscorrelation function 
%   of length L
%
%   Syntax
%   psi = rader(v,L)
%   psi = rader(v1,v2,L)
%
%   Description
% psi=rader(v,L) calculates the values of the autocorrelation function
%   for the sequence v(k)in the range la=-L(1)L.
% psi=rader(v1,v2,L) calculates the values of the crosscorrelation function
% for the two sequences v1 and v2 in the range la=-L(1)L
%
% Remark: For the calculation of the correlation function it has to be 
%         concernd that L is chosen sufficiently large to avoid 
%         Aliasing-errors.
%
%   References: [DSV1, chap. 3.3.4 and 3.3.5]
%
%   See also Matlab: xcorr

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek to H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Hans W. Schuessler and Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,3, nargin, 'struct'))
%
if length(v2)==1,
    L=v2;                     % Berechnung der Autokorrelierten
    M = 2*(L+1); mu = ((-1).^(0:(M-1)))'; v=v1(:);
    m = 1:L+1; V = fft(v(m),M); Psi = zeros(M,1);
    r = fix(length(v)/(L+1)) - 1;
    %
    for i = 1:r,
      vi = v(i*(L+1) + m); Vi = fft(vi,M);
      Wi = V + mu.*Vi;
      Psi = Psi + V.*conj(Wi); V = Vi;
    end
    psi1 = conj(ifft(Psi)/(r*(L+1)));
    if ~any(imag(v)),
      psi = real([psi1(L+1:-1:1); psi1(2:L+1)]);
    else
      psi = [conj(psi1(L+1:-1:1)); psi1(2:L+1)];
    end
else   % Berechnung der Kreuzkorrelierten
    nv1=length(v1); nv2=length(v2);
    if nv1~=nv2, 
      error('rader: unterschiedliche Lnge der Eingangsfolgen'); 
    end
    M=2*(L+1); m1=1:L+1; m2=1:2*(L+1); r=fix(nv1/(L+1))-1;
    Psi=zeros(M,1);
    %
    for i = 0:r-1;
      vi = v1(i*(L+1)+m1);  Vi=fft(vi,M);
      wi = v2(i*(L+1)+m2);  Wi=fft(wi,M);
      Psi = Psi + Vi.*conj(Wi);
    end
    psi1 = ifft(Psi)/(r*(L+1));
    psi = [psi1(L+2:1:M); conj(psi1(2:L+1))];
end
%% [EOF] - radar