function C=raychr(chi,alpha,tol)
%RAYCHR Characteristic function of a Rayleigh-distributed sequence
%
%   Syntax
%   C=raychr(chi)
%   C=raychr(chi,alpha)
%   C=raychr(chi,alpha,tol)
%
%   Description
% C = raychr(chi) calculates the characteristic function for the 
%   Rayleigh-distribution. The function is computed for the values given by
%   vector chi. By default, parameters are  set to alpha =1 and tol = 1.e-5.
% C = raychr(chi,alpha) calculates the characteristic function for 
%   selectable alpha.  
% C=raychr(chi,aplha,tol) In addition a tolerance value tol is defined for 
% 	interrupting the iterative determination of the real part of the 
%   characteristic function. The default value is tol = 1.e-5.
%
%   References: [DSV1, Chap. 3.1.3 p.145 ff],
%               [Gra06, Equation 3.952-8 and 9.210-1 ]
%
%   See also DSV-Lib: randray, rayvdf, rectchr, normchr, chi2chr, stuchr, 
%                     expchr, sinchr, binchr.

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek for H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,3, nargin, 'struct'))
%
if nargin<3, tol = 1.e-5; end
if nargin<2, alpha=1;     end
%
alp2=alpha*alpha; 
z=alp2/2*chi.*chi;
TOL=abs(chi); TOL(TOL<1)=1; TOL=tol*TOL; % Tolerance TOL(chi)
CR=F11(-0.5,0.5,z,TOL);
CI= chi.*alpha.*sqrt(2*pi).*exp(-alp2/2.*chi.^2)/2;
C=complex(CR,CI);
%
% ---- Helper-Function -----------------------------------------
function F=F11(al,ga,z,TOL)
% Confluent hypergeometric function 1F1(al,gy,z)
% Weighted by Cr0=exp(-z)
F=exp(-z); FF=F;
i=0;
while i >-1,
    FF=(al+i)/(ga+i)/(i+1)*FF.*z;
    if ~any(abs(FF)>TOL), i=-1; 
    elseif i<201,  F=F+FF; i=i+1;
    else   warning('DSVLIB:raychr',...
              'no convergence - reduce tolerance')
           i=-1;
    end   
end
%% [EOF] - raychr

