function [C,chi]=vdf2chr(pv,V)
%VDF2CHR Calculation of the characteristic function by DFT
%   The char. function is calculated in the range chi=[-chimax,chimax), 
%   using chi_max=pi/dV. Vector chi is linearly spaced. 
%
%   Syntax
%   C = vdf2chr(pv,V)
%   [C,chi]=vdf2chr(pv,V)
%
%   Description
% C=vdf2chr(pv,V) calculates the chararacteristic function for the
%   probability density function pv of a given random sequence.
%   The function pv is defined for the linearly spaced abscissa V.
%
% [C,chi]=vdf2chr(...) returns the frequency vector chi, optionally. 
%
%   References: [DSV1, Chap. 3.1.3]

%   See also DSV-Lib: rectchr, normchr, chi2chr, stuchr, raychr, 
%                     expchr, sinchr, binchr.
%

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek for H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,2,nargin, 'struct'))
%
Vmax=max(abs(V));
if ~all(abs(diff(diff(V/Vmax)))<10^2*eps),error('....'); end
dV=V(2)-V(1); chimax=pi/dV;
nV=length(V);
pv1=fftshift(pv);  pv1(pv1==Inf)=1.e24;
C=dV*conj(fft(pv1));
C=fftshift(C);
chi=linspace(-chimax,chimax,nV+1); 
chi=chi(1:nV);
%% [EOF] - vdf2chr
