function ssf=par2ssf(sop,sp)
%PAR2SSF - Transformation of standard parallel form into section in SSF form
% 
%   Syntax:
%   ssf=par2ssf(sop)
%   ssf=par2ssf(sop,sp)
%
%   Description
%
% ssf = [{A1} {B1} {Ct1} {D1}
%        ...
%        {An} {Bn} {Ctn} {Dn}
%                         d0]  % d0: coefficient of direct branch
% SOP   SOP coefficient array for parallel structure
% sp    additional scaling factors
%
%   References: [DSV1, chap. 5.2.2 and 7.1.3.2]
%               [Jack96]
%   See also DSV-Lib: sos2ssf, print_ssf
%            Matlab:  sos2ss

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,2, nargin, 'struct'))
nb=size(sop,1)-1;
if  sop(nb+1,2:3)~=0, error('PAR2PAR: error in SOP dataset'); end
if sop(:,3)==0, sop=par2par(sop); end  % set to parallel form of 1. kind
ds=sop(nb+1,1);
if nargin >1,
    [s1,s2,s3]=parse_sp(sp,nb);
    ss=[s2,s2,s2];
    sop(1:nb,1:3)=s1*s3*(ss.*sop(1:nb,1:3));
    ds = s1*s3*ds;
end
%
ssf=zeros(nb+1,6);
for n=1:nb,
   ssf1=sos2ssf(sop(n,:));
   ssf(n,:)=ssf1;                 
end
ssf(nb+1,4)= {ds};
%% Helper Function
function [s1,s2,s3]=parse_sp(sp,nb)
if ~isempty(sp)
    terr='PAR2PAR: mismatched scaling parameters';
  [Lsp,Csp]=size(sp);
  switch Csp,
  case 3
    if Lsp~=nb, error(terr); end
    s1=sp(1,1); s2=sp(1:nb,2); s3=sp(1,3);
  case 2
    if Lsp~=1, error(terr); end
    s1=sp(1,1); s2=ones(nb,1); s3=sp(1,2);
  case 1
    if Lsp~=nb, error(terr); end
    s1=1; s2=sp; s3=1;
  otherwise,    error(terr)
  end
else
    s1=1; s2=ones(nb,1); s3=1;
end
%% [EOF] - par2ssf
