function varargout=par2ssp(sop,sp)
%PAR2SSP Transformation of parallel structure in SOP-Form to SSP-Form
%   The function par2ssp first converts the second-order sections to a
%   zero-pole representation using sos2zp and then arranges the submatrices
%   to the SSP-form.
%
%   Syntax
%   [A,B,C,D]= par2ssp(sop)
%   [A,B,C,D]= par2ssp(sop,sp)
%       SSP  = par2ssp(...)
%
%   References: [DSV1, chap. 5.2.1, 5.2.2 and 7.1.3.&4.]
%
%   See also DSV-Lib: sos2ssc, par2ssf
%            Matlab:  sos2zp, zp2ss

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,2,nargin))
nb=size(sop,1)-1;
if nargin>1,        % Include scaling factors
   [s1,s2,s3]=parse_sp(sp,nb);  
else
   s1=1; s2=ones(nb,1); s3=1;
end
ss=[s2,s2,s2];
sop(1:nb,1:3)= s1*s3*(ss.*sop(1:nb,1:3)); 
sop(nb+1,1)  = s1*s3*sop(nb+1,1);
%% 
A = zeros(2*nb,2*nb); B=zeros(2*nb,1); C=zeros(1,2*nb); D=sop(nb+1,1);
ii=0;
for i=1:nb
    [z,p,k]= sos2zp(sop(i,:));
    [A1,B1,C1,D1] = zp2ss(z,p,k);
    if size(A1,1)==1, r=1; else r=2; end
    A(ii+1:ii+r,ii+1:ii+r) = A1;
    B(ii+1:ii+r,1)=B1;
    C(1,ii+1:ii+r)=C1;
    D= D+D1;
    ii=ii+r;
end
if ii<2*nb, 
    A=A(1:ii,1:ii); B=B(1:ii,1); C=C(1,1:ii);
end
if nargout>1,
    varargout{1}=A; varargout{2}=B; varargout{3}=C; varargout{4}=D;
else
    varargout{1}={A,B,C,D};
end

%% Helper Function
function [s1,s2,s3]=parse_sp(sp,nb)
if ~isempty(sp)
    terr='PAR2SSP: mismatched scaling parameters';
  [Lsp,Csp]=size(sp);
  switch Csp,
  case 3
    if Lsp~=nb, error(terr); end
    s1=sp(1,1); s2=sp(1:nb,2); s3=sp(1,3);
  case 2
    if Lsp~=1, error(terr); end
    s1=sp(1,1); s2=ones(nb,1); s3=sp(1,2);
  case 1
    if Lsp~=nb, error(terr); end
    s1=1; s2=sp; s3=1;
  otherwise,    error(terr)
  end
else
    s1=1; s2=ones(nb,1); s3=1;
end
%% [EOF] - par2ssp
