function [z,p,k]=par2zp(sop,sp)
%PAR2ZP Transformation of parallel to direct form in pole/zero description
%   The parallel form (sop) may be available in the 1. or 2. kind.
%  
% [z,p,k]=par2zp(sop)transforms the coefficient matrix (sop)of the parallel form 
%   back to the parameter z, p, k of the direct form.
% [z,p,k]=par2zp(sop,sp) multiplies optional scaling factors sp into the pure sop
%   Matrix and calculates the parameter of the direct form.
%   Remark: Scaling a parallel form for fixed point realization is not included 
%   in the current release of the DSV-Library
%
%   References: [DSV1, chap. 5.2]
%   See also DSV-Lib: zp2par, par2sos
%            Matlab:  sos2zp

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,2,nargin))
nb=size(sop,1)-1;
if nargin>1,
   [s1,s2,s3]=parse_sp(sp,nb);
else
   s1=1; s2=ones(nb,1); s3=1;
end
ss=[s2,s2,s2];
sop(1:nb,1:3)= s1*s3*(ss.*sop(1:nb,1:3)); 
sop(nb+1,1)  = s1*s3*sop(nb+1,1);
%% Build denominator
sop1=sop(1:nb,:);
[z,p]=sos2zp(sop1);      % Zeros are meaningless 
%% Build numerator
b=zeros(1,2*nb+1);
for ip=1:nb+1,
  bi=1.;
  for ii=1:nb,
  if ii~=ip, bi=conv(bi,sop(ii,4:6));end
  end
  if ip<=nb, b = b + conv(bi,sop(ip,1:3));
  else       b = b + sop(ip,1)*bi;
  end
end
b=b(1:size(p,1)+1);
k=b(1);
z=roots(b/k);
[z1,ind]=sort(real(z)); z=z(ind);
[p1,ind]=sort(real(p)); p=p(ind);
%% Helper Function
function [s1,s2,s3]=parse_sp(sp,nb)
if ~isempty(sp)
    terr='PAR2ZP: mismatched scaling parameters';
  [Lsp,Csp]=size(sp);
  switch Csp,
  case 3
    if Lsp~=nb, error(terr); end
    s1=sp(1,1); s2=sp(1:nb,2); s3=sp(1,3);
  case 2
    if Lsp~=1, error(terr); end
    s1=sp(1,1); s2=ones(nb,1); s3=sp(1,2);
  case 1
    if Lsp~=nb, error(terr); end
    s1=1; s2=sp; s3=1;
  otherwise,    error(terr)
  end
else
    s1=1; s2=ones(nb,1); s3=1;
end
%% [EOF] - par2zp
