function [Ac,Bc,Cc,Dc]=sos2ssc(sos,s)
%SOS2SSC Transformation of the SOS-form to state variable description SSC
%   In a first step the second order sections are transformed to state 
%   variable submatrices by the Matlab functions sos2ss(.). The
%   submatrices are allocated to the state variable description [Ac, Bc, Cc, Dc]
%   of the whole system, see [DSV1, (5.2.13)].
%
%   Syntax
%   [Ac,Bc,Cc,Dc]=sos2ssc(sos)
%   [Ac,Bc,Cc,Dc]=sos2ssc(sos,s)
%
%   Remark: For SOS with number of blocks > 1 the transformation by sos2ssc
%   is different to the Matlab function sos2ss.
%
%   References: [DSV1, chap. 5.2.1, 5.2.2 and 7.1.3.&4.]
%
%   See also DSV-Lib: sos2ss1, ssf2ssc
%            Matlab:  sos2ss, zp2sos

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,2,nargin))
%
nb=size(sos,1);
if nargin<2, s=1; end
s=s(:); Ls=length(s);
if     Ls==1, s=[s;ones(nb,1)];
elseif Ls==2, s=[s(1);ones(nb-1,1);s(2)];
elseif Ls~=nb+1, error('SOS2SSC: Wrong number of scaling parameters');
end
          % Allocation of cells allows blocks of first and second order
Ac=cell(nb,nb); Bc=cell(nb,1); Cc=cell(1,nb); 
Dd=[1 1]; %Ac(:,:)={A0};
for ir=1:nb
  [A,B,Ct,D] = sos2ss(sos(ir,1:6));
  B=s(ir)*B;  D=s(ir)*D;
  n=size(A,1);
  Ac(ir,ir)={A};      
   Bc(ir,1)={B*prod(Dd)};
  Cc(1,ir)={Ct};
  for i=1:ir-1
      Ac(ir,i)={B*Cc{1,i}*prod(Dd(1:ir-i))};
      Ac(i,ir)={zeros(size(Ac{i,ir-1},1),n)};
  end
  Dd=[1 D Dd(2:end)];
end
Dd=fliplr(Dd);
for ic=1:nb,
  Cc(1,ic)={prod(Dd(2+ic:end))*Cc{1,ic}};
end
Ac=cell2mat(Ac);
Bc=cell2mat(Bc);
Cc=s(nb+1)*cell2mat(Cc);
Dc=s(nb+1)*prod(Dd);
%% [EOF] -sos2ssc
