function [ssf,ss] = sos2ssf(sos,s,optstr)
%SOS2SSF Transform SOS-form to 2. order sections in state space form (SSF)
%   The SOS in standard MATLAB form are transformed to first or second order 
%   sections in state space sorms (SSF-Form) in optimal, normalized or 
%   canonical layout of the state matrix (A-Matrix).
% 
%   Syntax:
%   [ssf,ss] = sos2ssf(sos)
%   [ssf,ss] = sos2ssf(sos,s)
%   [ssf,ss] = sos2ssf(sos,s,optstr)
%
%   Description
%
%   ssf = [{A1} {B1} {Ct1} {D1}
%            ...
%          {An} {Bn} {Ctn} {Dn}]
%   ss =  additional gain factor
%   SOS   SOS coefficient matrix
%   g     additional scaling factor for the SOS structure
%
%   References: [DSV1, chap. 5.2.2, 6.3 and 7.1.7.6]
%               [Jack96, Deh03]
%   See also DSV-Lib: sos2ssc, print_ssf
%            Matlab:  sos2ss

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,3, nargin, 'struct'))
%
nb = size(sos,1);
if nargin>1, s0=prod(s); else s0=1; end
if nargin<3, optstr='optimal'; end
ssf=cell(nb,4);
%
switch optstr
%% Optimal State Space Structure
case 'optimal'
  for n=1:nb,
    % Build normed numerator and denominator 
                        % english notation (b0,b1,b2) /(1, a1,a2)
    b0 = sos(n,1); b1 = sos(n,2); b2 = sos(n,3);
     if b0 ~= 0, 
         b1 = b1/b0; b2 = b2/b0;
         s0 = s0*b0;    b0 = 1.;  D = 1.;
     else
         D = 0.; b0=0.;
     end
    a0 = sos(n,4); a1 = sos(n,5)/a0; a2=sos(n,6)/a0;
    s0 = s0/a0;   % a0=1.;
    if a2==0, % first order block, scaled by L2-Norm
       A = -a1;
       B = 1/sqrt(1+a1*a1);
       Ct= (b1-b0*a1)/B;
    else      % second order block, optimized
    % Build 2. order blocks in Optimal State Space Form
    % and independent L-2 Optimization for each block
    ga1= b1-b0*a1; ga2= b2-b0*a2;    
    %
    A11 = -a1/2;  A22=A11;
    K1  = ga2-a1*ga1/2;
    K2  = sqrt((ga2^2-ga1*ga2*a1+ga1^2*a2));
    DR = (1-a2)*((1+a2)^2-a1*a1); % Denominator of SOS Noise Power
    %
    if abs(ga1)>10*eps, 
        K1p2= (K1+K2)/ga1;  K1m2 = (K1-K2)/ga1;
        NRp=(1+(K1p2 + a1/2)^2)*(1+a2)-(2*K1p2+a1)*a1;
        NRm=(1+(K1m2 + a1/2)^2)*(1+a2)-(2*K1m2+a1)*a1;
        F  =sqrt(NRm/NRp)*sign(ga1);
        A12 = F * K1p2;
        A21 = K1m2/F;
        B1  = sqrt(DR/NRp);
        B2  = sign(ga1)*sqrt(DR/NRm);
        Ct1 = ga1/2*sqrt(NRp/DR);
        Ct2 = abs(ga1)/2*sqrt(NRm/DR);
    else
        if ga2>0,
            A12 =  sqrt(1+a1*a1/4-a1*a1/(1+a2));
            A21 = -(a2-a1*a1/4)/A12;
            B1 = 0.;
            B2 = sqrt(DR/((1+a1*a1/4)*(1+a2)-a1*a1));
            Ct1= ga2*sqrt((1+a2)/DR);
            Ct2 =0.;
        elseif ga2==0,
            A12 = sqrt(1+a1*a1/4-a1*a1/(1+a2));
            A21 = -(a2-a1*a1/4)/A12;
            B1  = sqrt(DR/((1+a1*a1/4)*(1+a2)-a1*a1))/2;
            B2  = B1;
            Ct1 = 0.;
            Ct2 = 0.;
        else
            A21 = -sqrt(1+a1*a1/4-a1*a1/(1+a2));
            A12 = -(a2-a1*a1/4)/A21;
            B1  = sqrt(DR/((1+a1*a1/4)*(1+a2)-a1*a1));   
            B2  = 0.;
            Ct1 = 0.;
            Ct2 = -ga2/sqrt(DR/(1+a2));
        end 
    end
    %
    A = [A11 A12;A21 A22];  B = [B1;B2];
    Ct  = [Ct1 Ct2];        % D set above
    %
    end
    ssf(n,:)=[{A} {B} {Ct} {D}];
  end
%% Normal State Space Structure
case 'normal'   % NORMAL Form	
  for n=1:nb,
   g0 = sos(n,1); g1 = sos(n,2); g2 = sos(n,3); a1 = sos(n,5); a2=sos(n,6);
   c1 = g1-g0*a1;      c2 = g2-g0*a2;
   if g2==0 && a2==0,
     A=-a1; B=1; Ct=c1; D=g0;
   else
     zp = roots(sos(n,4:6));
     p1r = real(zp(1));   p1i = imag(zp(1));
     if(abs(p1i)<eps), p1i = 100*eps; end
     gar = c1/2;   gai=-(c1*p1r+c2)/2/p1i;
     ga = complex(gar,gai);
     u  = sqrt(-j*ga);   
     b2 = real(u);       b1 = imag(u);
     c1 = -2*b2;     c2 = -2*b1;
     A = [p1r p1i; -p1i p1r]; B=[b1; b2]; Ct = [c1 c2]; D=g0;
   end
   ssf(n,:) = [{A} {B} {Ct} {D}];
  end
case 'ss1'  % State Variable Form 1
  for n=1:nb,
  b0 = sos(n,1); b1 = sos(n,2); b2 = sos(n,3); a1 = sos(n,5); a2=sos(n,6);
  c1=b1-b0*a1; c2=b2-b0*a2;
  A=[-a1,-a2; 1, 0]; B=[1; 0]; Ct= [c1 c2]; D=b0;    
  ssf(n,:) = [{A} {B} {Ct} {D}];    
  end
    otherwise
     error('SOS2SSF:Undefined form of structure')
end
ss=s0;
%% [EOF] - sos2ssf
