function varargout=ssf2ssp(ssf)
%SSF2SSP: Transformation of parallel structure SSF into the SSP-form
%    SSP: State Space form [A,B,C,D],  for Parallel structure
%
%   Syntax:
%   [A,B,C,D]= ssf2ssp(ssf)
%       SSP  = ssf2ssp(ssf)  % SSP Cell Array
%
%   Description:
%
%   ssf = [{A1} {B1} {Ct1} {D1}
%          ...
%          {An} {Bn} {Ctn} {Dn}
%                           d0]  % d0: coefficient of direct branch
%
%   SSP = {A,B,C,D}             
%
%   Example:  Output of  frequency response for SSF
%   To calculate the frequency response we first transform SSF to the state
%   space description using 'ssf2ssp' and get the matrix SSP. In a second
%   step we transform the state matrices to the transfer function using
%   'ss2tf'. Finally we calculate and plot the frequency response.
%
%       [A,B,C,D]=ssf2ssp(SSF);  % tally with SSP
%
%       [b,a]=ss2tf(A,B,C,D);
%       freqz(b,a);              % frequency response for SSF

%   See also DSV-Lib: par2ssp, par2ssf
%            Matlab:  ss2tf, freqz

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,1,nargin))
nb=size(ssf,1);
% check for SSF in parallel form
if  ~(isempty(ssf{nb,1})&& isempty(ssf{nb,2})&& isempty(ssf{nb,3})) , 
   error('SSF2SSP: Dataset is not a SSF parallel structure')
end
%% 
nb=nb-1;
A = zeros(2*nb,2*nb); B=zeros(2*nb,1); C=zeros(1,2*nb); D=ssf{nb+1,4};
ii=0;
for i=1:nb
    [A1,B1,C1,D1] = ssf_getblk(ssf,i);
    if size(A1,1)==1, r=1; else r=2; end
    A(ii+1:ii+r,ii+1:ii+r) = A1;
    B(ii+1:ii+r,1)=B1;
    C(1,ii+1:ii+r)=C1;
    D= D+D1;
    ii=ii+r;
end
if ii<2*nb, 
    A=A(1:ii,1:ii); B=B(1:ii,1); C=C(1,1:ii);
end
if nargout>1,
    varargout{1}=A; varargout{2}=B; varargout{3}=C; varargout{4}=D;
else
    varargout{1}={A,B,C,D};
end 
%% [EOF] - ssf2ssp
