function CAP=tf2catf(b,c,str)
%TF2CATF Transfer direct form to coupled allpass structure 
%  A system of odd order is transferred to a couples allpass with subsystems 
%  realized in direct form. The results are stored in the CAP (Coupled
%  Allpass Structure)
%
%   Syntax
%   CAP=tf2catf(b,c) 
%   CAP=tf2catf(b,c,str)
%
% Input parameters:
%  (b,c)    Coefficients in the direct form
%   str     Optional parameter to specify the substructure in use
%           ('df1', 'df1t', 'df2', 'df2t'; default: 'df1')
%
% Results: The results are stored in the CAP (Coupled Allpass Structure)
%   CAP.name    Coupled allpasses in direct form'
%   CAP.substr  Structure 'df1'
%   CAP.order   Filter order n
%   CAP.coef(1) Coefficient Matrix [b1;c1]
%   CAP.coef(2) Coefficient Matrix [b2;c2]
%
%   References: [DSV1, chap. 6.6.3 and 7.1.3]
%   See also DSV-Lib: compltf, euclidred, cap2tf, print_cap

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.01   Release 2010_2
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(2,3, nargin, 'struct'))
n=length(b)-1;
CAP=struct('name','Coupled allpass in direct form',...
    'substr','df1','order',[],'coef',[]);
if nargin>2, 
    if ~any(strcmp(str,{'df1','df1t','df2','df2t'})),
        error ('TF2CATF: Not allowed filter structure'); end
    CAP.substr=str; 
end
%% Calculation of complementary function Z2
Z1=b;
Z2 = compltf(Z1,c);
if rem(n,2)~=0,
  bs1=Z1+Z2;
else
  warning('DSVLib:tf2catf',' Filter order even')
  bs1=Z1-1i*Z2;
end
  [~,ca1,c0] = euclidred(bs1,c);
  ca11=ca1(1);  
  den1=ca1/ca11;     den2=c0*ca11;
  num1=fliplr(conj(den1)); num2=fliplr(conj(den2));
  %%  Check Transfer function
  bca = (conv(num1,den2)+conv(num2,den1))/2;
  if abs(bca - b) > 1e4*eps,
      warning('DSVLib:tf2catf',...
          ' Filter can not be realized as coupled allpass')
  end
  %
  CAP.order=n;
  CAP.coef{1}=[num1;den1];   
  CAP.coef{2}=[num2;den2];
  CAP.coef{3}=0.5;
%% [EOF] - tf2catf
