function sop=zp2par(z,p,k)
% ZP2PAR: Conversion of zero-pole description to parallel form of 2. kind
%   For the parallel form of 2. kind the direct throughput is distributed over
%   all parallel sections, see [DSV1, Chapt. 5.2.2]
%
%   Syntax
%   sop=zp2par(z,p,k)
%
%   Description
% sop = zp2par(z,p,k) calculates the coefficients of the parallel form
%   (2. kind) for a system defined by zero locations in vector z, pole
%   locations in vector p, and a gain factor in scalar k. The SOP-matrix is
%   returned. 
%   For the definition of the SOP-matrix see [DSV1 Chap. 5.2].
%
%   The residues in discrete time-domain are calculated here in a modified
%   form compared to residuez(.) in order to avoid the roundabout way via 
%   building the transfer function and again root finding.
%
%   References: [DSV1, chap. 5.2 and 7.1.3.2]
%
%   See also DSV-Lib: par2par, tf2par
%            Matlab:  mpoles {get info by 'help mpoles'}, deconv, poly,
%                     polyval, residuez

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(3,3,nargin))
mpoles_tol = 1e-5;
mul=mpoles(p,mpoles_tol);
if any(mul>1), error('ZP2PAR: System owns multiple poles'); end 
% Simultaneous calculation of residues and numerator coefficients of
% parallel blocks;
Lz=length(z); Lp=length(p);
if Lz > Lp, error('ZP2PAR: Numerator order > denominator order'); end
pp=p;   % sort poles in conjungate complex and real poles
[pr,idx] = sort(real(pp));      
p=pp(idx);
[pa,idx]=sort(-abs(p));
p=p(idx);
% Calculation  direct branch of parallel form
b=real(k*poly(z));
a=real(poly(pp));
if Lz==Lp,
    [d,bflip] = deconv(fliplr(b),fliplr(a));
    bflip(1)=[];
else
    d=0;
    bflip=fliplr(b);
end
i=0; j=1;  nb=ceil(Lp/2);  sop=zeros(nb,6);
while i<Lp,
    i=i+1;
    pr= real(p(i));  ppi=imag(p(i));      % calc residue fpr p(i)
    r = polyval(bflip,1/p(i)) ./ blpolyval(p([1:(i-1),(i+1):Lp]),1/p(i)) ;    
      rr = real(r); ri=imag(r);
    if i<Lp && abs(1 - real(p(i+1)/pr)) < 1e5*eps,   % conj complex poles
      if abs(1+imag(p(i+1))/ppi)>1e5*eps, 
               error ('ZP2PAR: not comp. poles'); end
      sop(j,1:3) = [2*rr  -2*(rr*pr+ri*ppi)  0]; 
      sop(j,4:6) = [ 1  -2*pr  pr*pr+ppi*ppi];   
      i=i+1;
    else
      sop(j,1:3) = [rr 0 0];                      % real poles
      sop(j,4:6) = [ 1  -pr  0];                     
    end
    j=j+1;
end
sop=[sop;[d 0 0 0 0 0]];
end
%% Helper Function
function y=blpolyval(p,z)
% calculation of polynom by separation into blocks of second order
y=1;
np=length(p);
i=0;
while i<np
   i=i+1;
   pr= real(p(i));
   if i<np && abs(1 - real(p(i+1)/pr)) < 1e5*eps,         %  conjungate complex
      ppi = imag(p(i));
      if abs(1+imag(p(i+1))/ppi)>1e5*eps, 
         error ('zp2par: not comp. pole'); end
      temp=fliplr(poly( p(i:i+1)) );
      i=i+1;
   else
      temp=fliplr(poly( p(i)) );
   end
   y = y * polyval(temp,z);
end
end
%% [EOF] - zp2par
   
