function [pas,GM]=syspassiv(A,tol)
%SYSPASSIV Cecks for passivity or pseudo passivity
%
%   Syntax
%   pas = syspassiv(A)
%   pas = syspassiv(A,tol)
%   [pas,GM]=syspassiv(...)
%
%   pas=0  System not passiv
%      =1  System passiv
%      =2  System pseudo passiv
%
%   The tolerance for the pseudo passivity test can be tuned by the 
%   parameter tol. By default tol is set to 1.e-8.
%   For the pseudo passivity test the assessment matrix G= diag(1,g2,g3,....)
%   is optimized by the search function fminsearch(:).
%
%   The final matrix GM=G is optionally returned.
%
%   References: [DSV1, chap. 5.6.3 and 6.5.3]
%               [Fet86],[MeW75],[Mee75]
%
%   See also DSV-Lib: isposdef, systable
%            Matlab:  fminsearch, eig, rank

%%  ---------------------------------------------------------------
%   DSV1-Bibliothek zu H.W. Schuessler: Digitale Signalverarbeitung 1,
%                      Springer (2008)
%   Authors: Guenter F. Dehner
%   Version: 1.0
%   Copyright 2007 by authors - not released for commercial use
%%  ---------------------------------------------------------------
error(nargchk(1,2, nargin, 'struct'))
%
if nargin<2, tol=1.e-8; end
n=length(A);
%% General passivity test using G=eye(n)
Q  = eye(n)-A'*A;
[L,r]=isposdef(Q);
if L==0 && r<n,     % check if positiv semi definite
   e=sort(eig(Q),'descend');
   L=all(e(1:r)>eps) && all(abs(e(n-r+1:n))<eps);
end
if L==1, 
   pas=double(L); 
   if nargout>1,GM=eye(n); end
   return
end
%% Pseudo passivity test 
x0=ones(1,n-1);  % g2, g3, ... =1  
options = optimset('TolX',1.e-6);
% search for suitable test matrix G by optimization algorithm
[g2_,fval,exitf,output]=fminsearch(@stab,x0,options,A);
% output.message
G  = diag([1,g2_]);
Q  = G - A'*G*A;
d=eig(Q);
L=all(d>eps);
if ~L,
  dd=find(abs(d)<tol); nz=length(dd);
  if nz~=0,  
  L=(n-rank(Q,tol)==nz);
  end
end
pas=2*(L>0);
if nargout>1,GM=G; end
% --------------------------------------------------------------
%% Function for optimization
function fd=stab(x,A)
G  = diag([1,x]);
Q  = G - A'*G*A;
d=eig(Q);  
fd=-min(real(d(:)));
%% [EOF] - syspassiv
